
/* ==========================================================================
   CSS3 包括的サンプル - Tree-sitter Analyzer テスト用
   ========================================================================== */

/* カスタムプロパティ（CSS変数） */
:root {
    /* カラーパレット */
    --primary-color: #007bff;
    --secondary-color: #6c757d;
    --success-color: #28a745;
    --danger-color: #dc3545;
    --warning-color: #ffc107;
    --info-color: #17a2b8;
    --light-color: #f8f9fa;
    --dark-color: #343a40;

    /* タイポグラフィ */
    --font-family-sans: 'Helvetica Neue', Arial, 'Hiragino Kaku Gothic ProN', 'Hiragino Sans', Meiryo, sans-serif;
    --font-family-serif: Georgia, 'Times New Roman', 'Yu Mincho', 'Hiragino Mincho ProN', serif;
    --font-family-mono: 'SF Mono', Monaco, 'Cascadia Code', 'Roboto Mono', Consolas, 'Courier New', monospace;

    /* サイズ */
    --font-size-xs: 0.75rem;
    --font-size-sm: 0.875rem;
    --font-size-base: 1rem;
    --font-size-lg: 1.125rem;
    --font-size-xl: 1.25rem;
    --font-size-2xl: 1.5rem;
    --font-size-3xl: 1.875rem;
    --font-size-4xl: 2.25rem;

    /* スペーシング */
    --spacing-xs: 0.25rem;
    --spacing-sm: 0.5rem;
    --spacing-md: 1rem;
    --spacing-lg: 1.5rem;
    --spacing-xl: 3rem;

    /* ブレークポイント */
    --breakpoint-sm: 576px;
    --breakpoint-md: 768px;
    --breakpoint-lg: 992px;
    --breakpoint-xl: 1200px;

    /* アニメーション */
    --transition-fast: 0.15s ease-in-out;
    --transition-base: 0.3s ease-in-out;
    --transition-slow: 0.5s ease-in-out;

    /* シャドウ */
    --shadow-sm: 0 1px 2px 0 rgba(0, 0, 0, 0.05);
    --shadow-base: 0 1px 3px 0 rgba(0, 0, 0, 0.1), 0 1px 2px 0 rgba(0, 0, 0, 0.06);
    --shadow-lg: 0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05);

    /* ボーダー半径 */
    --border-radius-sm: 0.125rem;
    --border-radius-base: 0.25rem;
    --border-radius-lg: 0.5rem;
    --border-radius-full: 9999px;
}

/* ==========================================================================
   リセット・ノーマライズ
   ========================================================================== */

/* ボックスサイジングのリセット */
*,
*::before,
*::after {
    box-sizing: border-box;
}

/* マージンとパディングのリセット */
* {
    margin: 0;
    padding: 0;
}

/* HTML5要素のブロック表示 */
article,
aside,
details,
figcaption,
figure,
footer,
header,
hgroup,
main,
nav,
section,
summary {
    display: block;
}

/* ==========================================================================
   基本スタイル
   ========================================================================== */

/* ドキュメント全体 */
html {
    font-size: 16px;
    line-height: 1.6;
    scroll-behavior: smooth;
    -webkit-text-size-adjust: 100%;
    -ms-text-size-adjust: 100%;
}

/* ボディ */
body {
    font-family: var(--font-family-sans);
    font-size: var(--font-size-base);
    line-height: 1.6;
    color: var(--dark-color);
    background-color: #ffffff;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}

/* ==========================================================================
   タイポグラフィ
   ========================================================================== */

/* 見出し */
h1, h2, h3, h4, h5, h6 {
    font-family: var(--font-family-sans);
    font-weight: 700;
    line-height: 1.2;
    margin-bottom: var(--spacing-md);
    color: var(--dark-color);
}

h1 {
    font-size: var(--font-size-4xl);
    margin-bottom: var(--spacing-lg);
}

h2 {
    font-size: var(--font-size-3xl);
}

h3 {
    font-size: var(--font-size-2xl);
}

h4 {
    font-size: var(--font-size-xl);
}

h5 {
    font-size: var(--font-size-lg);
}

h6 {
    font-size: var(--font-size-base);
}

/* 段落 */
p {
    margin-bottom: var(--spacing-md);
    line-height: 1.7;
}

/* 強調 */
strong,
b {
    font-weight: 700;
}

em,
i {
    font-style: italic;
}

/* マーク */
mark {
    background-color: var(--warning-color);
    color: var(--dark-color);
    padding: 0.1em 0.2em;
    border-radius: var(--border-radius-sm);
}

/* コード */
code,
kbd,
pre,
samp {
    font-family: var(--font-family-mono);
    font-size: 0.9em;
}

code {
    background-color: var(--light-color);
    color: var(--danger-color);
    padding: 0.2em 0.4em;
    border-radius: var(--border-radius-sm);
}

pre {
    background-color: var(--light-color);
    color: var(--dark-color);
    padding: var(--spacing-md);
    border-radius: var(--border-radius-base);
    overflow-x: auto;
    line-height: 1.4;
}

pre code {
    background-color: transparent;
    color: inherit;
    padding: 0;
}

/* 引用 */
blockquote {
    margin: var(--spacing-lg) 0;
    padding: var(--spacing-md) var(--spacing-lg);
    border-left: 4px solid var(--primary-color);
    background-color: var(--light-color);
    font-style: italic;
}

blockquote p:last-child {
    margin-bottom: 0;
}

blockquote footer {
    margin-top: var(--spacing-sm);
    font-size: var(--font-size-sm);
    color: var(--secondary-color);
}

blockquote footer::before {
    content: "— ";
}

/* ==========================================================================
   リンク
   ========================================================================== */

a {
    color: var(--primary-color);
    text-decoration: none;
    transition: color var(--transition-fast);
}

a:hover,
a:focus {
    color: #0056b3;
    text-decoration: underline;
}

a:focus {
    outline: 2px solid var(--primary-color);
    outline-offset: 2px;
}

/* 外部リンク */
a[target="_blank"]::after {
    content: " ↗";
    font-size: 0.8em;
    opacity: 0.7;
}

/* ==========================================================================
   リスト
   ========================================================================== */

ul,
ol {
    margin-bottom: var(--spacing-md);
    padding-left: var(--spacing-lg);
}

li {
    margin-bottom: var(--spacing-xs);
}

/* ネストしたリスト */
ul ul,
ol ol,
ul ol,
ol ul {
    margin-bottom: 0;
    margin-top: var(--spacing-xs);
}

/* 定義リスト */
dl {
    margin-bottom: var(--spacing-md);
}

dt {
    font-weight: 700;
    margin-bottom: var(--spacing-xs);
}

dd {
    margin-bottom: var(--spacing-sm);
    margin-left: var(--spacing-lg);
}

/* ==========================================================================
   テーブル
   ========================================================================== */

table {
    width: 100%;
    border-collapse: collapse;
    margin-bottom: var(--spacing-lg);
    background-color: #ffffff;
    box-shadow: var(--shadow-base);
    border-radius: var(--border-radius-base);
    overflow: hidden;
}

caption {
    padding: var(--spacing-md);
    font-weight: 700;
    text-align: left;
    background-color: var(--light-color);
}

th,
td {
    padding: var(--spacing-sm) var(--spacing-md);
    text-align: left;
    border-bottom: 1px solid #dee2e6;
}

th {
    background-color: var(--light-color);
    font-weight: 700;
    color: var(--dark-color);
}

tbody tr:hover {
    background-color: rgba(0, 123, 255, 0.05);
}

/* テーブルの特殊セル */
.frequency.high {
    color: var(--success-color);
    font-weight: 700;
}

.frequency.medium {
    color: var(--warning-color);
    font-weight: 700;
}

.frequency.low {
    color: var(--secondary-color);
    font-weight: 700;
}

/* ==========================================================================
   フォーム
   ========================================================================== */

/* フィールドセット */
fieldset {
    border: 1px solid #dee2e6;
    border-radius: var(--border-radius-base);
    padding: var(--spacing-lg);
    margin-bottom: var(--spacing-lg);
}

legend {
    font-weight: 700;
    padding: 0 var(--spacing-sm);
    color: var(--dark-color);
}

/* フォームグループ */
.form-group {
    margin-bottom: var(--spacing-md);
}

/* ラベル */
label {
    display: block;
    margin-bottom: var(--spacing-xs);
    font-weight: 600;
    color: var(--dark-color);
}

.required {
    color: var(--danger-color);
}

/* 入力フィールド */
input[type="text"],
input[type="email"],
input[type="tel"],
input[type="url"],
input[type="password"],
input[type="number"],
input[type="date"],
input[type="time"],
input[type="datetime-local"],
input[type="search"],
textarea,
select {
    width: 100%;
    padding: var(--spacing-sm) var(--spacing-md);
    border: 1px solid #ced4da;
    border-radius: var(--border-radius-base);
    font-size: var(--font-size-base);
    line-height: 1.5;
    transition: border-color var(--transition-fast), box-shadow var(--transition-fast);
    background-color: #ffffff;
}

input:focus,
textarea:focus,
select:focus {
    outline: none;
    border-color: var(--primary-color);
    box-shadow: 0 0 0 3px rgba(0, 123, 255, 0.1);
}

input:invalid,
textarea:invalid,
select:invalid {
    border-color: var(--danger-color);
}

/* プレースホルダー */
::placeholder {
    color: var(--secondary-color);
    opacity: 1;
}

/* テキストエリア */
textarea {
    resize: vertical;
    min-height: 100px;
}

/* セレクト */
select {
    cursor: pointer;
    background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' fill='none' viewBox='0 0 20 20'%3e%3cpath stroke='%236b7280' stroke-linecap='round' stroke-linejoin='round' stroke-width='1.5' d='m6 8 4 4 4-4'/%3e%3c/svg%3e");
    background-position: right 0.5rem center;
    background-repeat: no-repeat;
    background-size: 1.5em 1.5em;
    padding-right: 2.5rem;
}

/* ラジオボタンとチェックボックス */
.radio-group,
.checkbox-group {
    display: flex;
    flex-wrap: wrap;
    gap: var(--spacing-md);
}

input[type="radio"],
input[type="checkbox"] {
    width: auto;
    margin-right: var(--spacing-xs);
}

/* レンジスライダー */
input[type="range"] {
    width: 100%;
    height: 6px;
    border-radius: var(--border-radius-full);
    background: #ddd;
    outline: none;
    -webkit-appearance: none;
}

input[type="range"]::-webkit-slider-thumb {
    -webkit-appearance: none;
    appearance: none;
    width: 20px;
    height: 20px;
    border-radius: 50%;
    background: var(--primary-color);
    cursor: pointer;
}

input[type="range"]::-moz-range-thumb {
    width: 20px;
    height: 20px;
    border-radius: 50%;
    background: var(--primary-color);
    cursor: pointer;
    border: none;
}

/* ファイル入力 */
input[type="file"] {
    padding: var(--spacing-xs);
    border: 2px dashed #ced4da;
    border-radius: var(--border-radius-base);
    background-color: var(--light-color);
}

input[type="file"]:hover {
    border-color: var(--primary-color);
}

/* ==========================================================================
   ボタン
   ========================================================================== */

.btn {
    display: inline-block;
    padding: var(--spacing-sm) var(--spacing-md);
    font-size: var(--font-size-base);
    font-weight: 600;
    line-height: 1.5;
    text-align: center;
    text-decoration: none;
    vertical-align: middle;
    cursor: pointer;
    border: 1px solid transparent;
    border-radius: var(--border-radius-base);
    transition: all var(--transition-fast);
    user-select: none;
}

.btn:focus {
    outline: none;
    box-shadow: 0 0 0 3px rgba(0, 123, 255, 0.25);
}

.btn:disabled {
    opacity: 0.6;
    cursor: not-allowed;
}

/* ボタンバリエーション */
.btn-primary {
    color: #ffffff;
    background-color: var(--primary-color);
    border-color: var(--primary-color);
}

.btn-primary:hover {
    background-color: #0056b3;
    border-color: #004085;
}

.btn-secondary {
    color: #ffffff;
    background-color: var(--secondary-color);
    border-color: var(--secondary-color);
}

.btn-secondary:hover {
    background-color: #545b62;
    border-color: #4e555b;
}

.btn-success {
    color: #ffffff;
    background-color: var(--success-color);
    border-color: var(--success-color);
}

.btn-danger {
    color: #ffffff;
    background-color: var(--danger-color);
    border-color: var(--danger-color);
}

/* ボタンサイズ */
.btn-sm {
    padding: var(--spacing-xs) var(--spacing-sm);
    font-size: var(--font-size-sm);
}

.btn-lg {
    padding: var(--spacing-md) var(--spacing-lg);
    font-size: var(--font-size-lg);
}

/* ==========================================================================
   レイアウト
   ========================================================================== */

/* コンテナ */
.container {
    width: 100%;
    max-width: 1200px;
    margin: 0 auto;
    padding: 0 var(--spacing-md);
}

/* グリッドシステム */
.grid {
    display: grid;
    gap: var(--spacing-md);
}

.grid-cols-1 {
    grid-template-columns: 1fr;
}

.grid-cols-2 {
    grid-template-columns: repeat(2, 1fr);
}

.grid-cols-3 {
    grid-template-columns: repeat(3, 1fr);
}

.grid-cols-4 {
    grid-template-columns: repeat(4, 1fr);
}

/* Flexboxユーティリティ */
.flex {
    display: flex;
}

.flex-col {
    flex-direction: column;
}

.flex-wrap {
    flex-wrap: wrap;
}

.items-center {
    align-items: center;
}

.justify-center {
    justify-content: center;
}

.justify-between {
    justify-content: space-between;
}

/* ==========================================================================
   ヘッダー・ナビゲーション
   ========================================================================== */

.main-header {
    background-color: #ffffff;
    box-shadow: var(--shadow-base);
    position: sticky;
    top: 0;
    z-index: 1000;
}

.navigation {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: var(--spacing-md) 0;
}

.nav-brand h1 {
    margin: 0;
    font-size: var(--font-size-xl);
}

.nav-brand a {
    color: var(--dark-color);
    text-decoration: none;
}

.nav-menu {
    display: flex;
    list-style: none;
    margin: 0;
    padding: 0;
    gap: var(--spacing-lg);
}

.nav-link {
    color: var(--dark-color);
    text-decoration: none;
    font-weight: 500;
    padding: var(--spacing-xs) var(--spacing-sm);
    border-radius: var(--border-radius-base);
    transition: background-color var(--transition-fast);
}

.nav-link:hover {
    background-color: var(--light-color);
    text-decoration: none;
}

/* ハンバーガーメニュー */
.menu-toggle {
    display: none;
    flex-direction: column;
    background: none;
    border: none;
    cursor: pointer;
    padding: var(--spacing-xs);
}

.menu-toggle span {
    width: 25px;
    height: 3px;
    background-color: var(--dark-color);
    margin: 3px 0;
    transition: var(--transition-fast);
}

/* ==========================================================================
   メインコンテンツ
   ========================================================================== */

.main-content {
    min-height: calc(100vh - 200px);
}

/* ヒーローセクション */
.hero-section {
    background: linear-gradient(135deg, var(--primary-color) 0%, #0056b3 100%);
    color: #ffffff;
    padding: var(--spacing-xl) 0;
    text-align: center;
}

.hero-title {
    font-size: var(--font-size-4xl);
    margin-bottom: var(--spacing-md);
    text-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
}

.hero-subtitle {
    font-size: var(--font-size-xl);
    margin-bottom: var(--spacing-lg);
    opacity: 0.9;
}

.hero-actions {
    display: flex;
    gap: var(--spacing-md);
    justify-content: center;
    flex-wrap: wrap;
}

/* セクション */
section {
    padding: var(--spacing-xl) 0;
}

section:nth-child(even) {
    background-color: var(--light-color);
}

.section-subtitle {
    font-size: var(--font-size-lg);
    color: var(--secondary-color);
    margin-bottom: var(--spacing-lg);
}

/* 機能グリッド */
.features-grid {
    display: grid;
    grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
    gap: var(--spacing-lg);
    margin-top: var(--spacing-lg);
}

.feature-card {
    background-color: #ffffff;
    padding: var(--spacing-lg);
    border-radius: var(--border-radius-lg);
    box-shadow: var(--shadow-base);
    transition: transform var(--transition-base), box-shadow var(--transition-base);
}

.feature-card:hover {
    transform: translateY(-5px);
    box-shadow: var(--shadow-lg);
}

.feature-card h3 {
    color: var(--primary-color);
    margin-bottom: var(--spacing-md);
}

/* リストグリッド */
.lists-grid {
    display: grid;
    grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
    gap: var(--spacing-lg);
}

.list-item {
    background-color: #ffffff;
    padding: var(--spacing-lg);
    border-radius: var(--border-radius-base);
    box-shadow: var(--shadow-base);
}

/* メディアグリッド */
.media-grid {
    display: grid;
    grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
    gap: var(--spacing-lg);
}

.media-item {
    text-align: center;
}

.media-item img,
.media-item video,
.media-item audio {
    max-width: 100%;
    height: auto;
    border-radius: var(--border-radius-base);
}

figcaption {
    margin-top: var(--spacing-sm);
    font-size: var(--font-size-sm);
    color: var(--secondary-color);
}

/* ==========================================================================
   サイドバー
   ========================================================================== */

.sidebar {
    background-color: var(--light-color);
    padding: var(--spacing-lg);
    border-radius: var(--border-radius-base);
    margin-top: var(--spacing-lg);
}

.widget {
    margin-bottom: var(--spacing-lg);
}

.widget:last-child {
    margin-bottom: 0;
}

.widget h3 {
    margin-bottom: var(--spacing-md);
    color: var(--dark-color);
    border-bottom: 2px solid var(--primary-color);
    padding-bottom: var(--spacing-xs);
}

.widget ul {
    list-style: none;
    padding: 0;
}

.widget li {
    margin-bottom: var(--spacing-xs);
}

.news-item {
    padding: var(--spacing-sm);
    background-color: #ffffff;
    border-radius: var(--border-radius-base);
    border-left: 4px solid var(--info-color);
}

.news-item time {
    font-size: var(--font-size-sm);
    color: var(--secondary-color);
    font-weight: 600;
}

/* ==========================================================================
   フッター
   ========================================================================== */

.main-footer {
    background-color: var(--dark-color);
    color: #ffffff;
    padding: var(--spacing-xl) 0 var(--spacing-md);
    margin-top: var(--spacing-xl);
}

.footer-content {
    display: grid;
    grid-template-columns: repeat(auto-fit, minmax(250px, 1fr));
    gap: var(--spacing-lg);
    margin-bottom: var(--spacing-lg);
}

.footer-section h4 {
    color: #ffffff;
    margin-bottom: var(--spacing-md);
}

.footer-section ul {
    list-style: none;
    padding: 0;
}

.footer-section li {
    margin-bottom: var(--spacing-xs);
}

.footer-section a {
    color: #adb5bd;
    text-decoration: none;
    transition: color var(--transition-fast);
}

.footer-section a:hover {
    color: #ffffff;
}

.footer-bottom {
    border-top: 1px solid #495057;
    padding-top: var(--spacing-md);
    text-align: center;
    color: #adb5bd;
}

/* ==========================================================================
   アニメーション
   ========================================================================== */

/* フェードイン */
@keyframes fadeIn {
    from {
        opacity: 0;
        transform: translateY(20px);
    }
    to {
        opacity: 1;
        transform: translateY(0);
    }
}

.fade-in {
    animation: fadeIn 0.6s ease-out;
}

/* スライドイン */
@keyframes slideInLeft {
    from {
        opacity: 0;
        transform: translateX(-30px);
    }
    to {
        opacity: 1;
        transform: translateX(0);
    }
}

.slide-in-left {
    animation: slideInLeft 0.5s ease-out;
}

/* パルス */
@keyframes pulse {
    0%, 100% {
        transform: scale(1);
    }
    50% {
        transform: scale(1.05);
    }
}

.pulse {
    animation: pulse 2s infinite;
}

/* スピン */
@keyframes spin {
    from {
        transform: rotate(0deg);
    }
    to {
        transform: rotate(360deg);
    }
}

.spin {
    animation: spin 1s linear infinite;
}

/* バウンス */
@keyframes bounce {
    0%, 20%, 53%, 80%, 100% {
        transform: translate3d(0, 0, 0);
    }
    40%, 43% {
        transform: translate3d(0, -30px, 0);
    }
    70% {
        transform: translate3d(0, -15px, 0);
    }
    90% {
        transform: translate3d(0, -4px, 0);
    }
}

.bounce {
    animation: bounce 1s ease infinite;
}

/* ==========================================================================
   ユーティリティクラス
   ========================================================================== */

/* 表示・非表示 */
.hidden {
    display: none !important;
}

.visible {
    visibility: visible !important;
}

.invisible {
    visibility: hidden !important;
}

/* テキスト配置 */
.text-left {
    text-align: left !important;
}

.text-center {
    text-align: center !important;
}

.text-right {
    text-align: right !important;
}

.text-justify {
    text-align: justify !important;
}

/* テキストカラー */
.text-primary {
    color: var(--primary-color) !important;
}

.text-secondary {
    color: var(--secondary-color) !important;
}

.text-success {
    color: var(--success-color) !important;
}

.text-danger {
    color: var(--danger-color) !important;
}

.text-warning {
    color: var(--warning-color) !important;
}

.text-info {
    color: var(--info-color) !important;
}

.text-light {
    color: var(--light-color) !important;
}

.text-dark {
    color: var(--dark-color) !important;
}

.text-white {
    color: #ffffff !important;
}

/* 背景色 */
.bg-primary {
    background-color: var(--primary-color) !important;
}

.bg-secondary {
    background-color: var(--secondary-color) !important;
}

.bg-success {
    background-color: var(--success-color) !important;
}

.bg-danger {
    background-color: var(--danger-color) !important;
}

.bg-warning {
    background-color: var(--warning-color) !important;
}

.bg-info {
    background-color: var(--info-color) !important;
}

.bg-light {
    background-color: var(--light-color) !important;
}

.bg-dark {
    background-color: var(--dark-color) !important;
}

.bg-white {
    background-color: #ffffff !important;
}

/* マージン・パディング */
.m-0 { margin: 0 !important; }
.m-1 { margin: var(--spacing-xs) !important; }
.m-2 { margin: var(--spacing-sm) !important; }
.m-3 { margin: var(--spacing-md) !important; }
.m-4 { margin: var(--spacing-lg) !important; }
.m-5 { margin: var(--spacing-xl) !important; }

.p-0 { padding: 0 !important; }
.p-1 { padding: var(--spacing-xs) !important; }
.p-2 { padding: var(--spacing-sm) !important; }
.p-3 { padding: var(--spacing-md) !important; }
.p-4 { padding: var(--spacing-lg) !important; }
.p-5 { padding: var(--spacing-xl) !important; }

/* ボーダー */
.border {
    border: 1px solid #dee2e6 !important;
}

.border-top {
    border-top: 1px solid #dee2e6 !important;
}

.border-bottom {
    border-bottom: 1px solid #dee2e6 !important;
}

.border-left {
    border-left: 1px solid #dee2e6 !important;
}

.border-right {
    border-right: 1px solid #dee2e6 !important;
}

.border-0 {
    border: 0 !important;
}

/* ボーダー半径 */
.rounded {
    border-radius: var(--border-radius-base) !important;
}

.rounded-sm {
    border-radius: var(--border-radius-sm) !important;
}

.rounded-lg {
    border-radius: var(--border-radius-lg) !important;
}

.rounded-full {
    border-radius: var(--border-radius-full) !important;
}

/* シャドウ */
.shadow {
    box-shadow: var(--shadow-base) !important;
}

.shadow-sm {
    box-shadow: var(--shadow-sm) !important;
}

.shadow-lg {
    box-shadow: var(--shadow-lg) !important;
}

.shadow-none {
    box-shadow: none !important;
}

/* ==========================================================================
   レスポンシブデザイン
   ========================================================================== */

/* スマートフォン（縦向き） */
@media (max-width: 575.98px) {
    .container {
        padding: 0 var(--spacing-sm);
    }

    .hero-title {
        font-size: var(--font-size-2xl);
    }

    .hero-subtitle {
        font-size: var(--font-size-lg);
    }

    .hero-actions {
        flex-direction: column;
        align-items: center;
    }

    .features-grid,
    .lists-grid,
    .media-grid {
        grid-template-columns: 1fr;
    }

    .nav-menu {
        display: none;
        position: absolute;
        top: 100%;
        left: 0;
        right: 0;
        background-color: #ffffff;
        flex-direction: column;
        padding: var(--spacing-md);
        box-shadow: var(--shadow-base);
    }

    .nav-menu.active {
        display: flex;
    }

    .menu-toggle {
        display: flex;
    }

    .footer-content {
        grid-template-columns: 1fr;
        text-align: center;
    }

    table {
        font-size: var(--font-size-sm);
    }

    th,
    td {
        padding: var(--spacing-xs) var(--spacing-sm);
    }
}

/* タブレット */
@media (min-width: 576px) and (max-width: 991.98px) {
    .features-grid {
        grid-template-columns: repeat(2, 1fr);
    }

    .lists-grid {
        grid-template-columns: repeat(2, 1fr);
    }

    .media-grid {
        grid-template-columns: repeat(2, 1fr);
    }
}

/* デスクトップ */
@media (min-width: 992px) {
    .sidebar {
        position: sticky;
        top: 100px;
        align-self: start;
    }

    .main-content {
        display: grid;
        grid-template-columns: 1fr 300px;
        gap: var(--spacing-xl);
        align-items: start;
    }
}

/* 大画面 */
@media (min-width: 1200px) {
    .container {
        max-width: 1400px;
    }

    .features-grid {
        grid-template-columns: repeat(3, 1fr);
    }
}

/* ダークモード */
@media (prefers-color-scheme: dark) {
    :root {
        --dark-color: #f8f9fa;
        --light-color: #343a40;
    }

    body {
        background-color: #121212;
        color: var(--dark-color);
    }

    .main-header {
        background-color: #1e1e1e;
    }

    .feature-card,
    .list-item {
        background-color: #1e1e1e;
        color: var(--dark-color);
    }

    table {
        background-color: #1e1e1e;
    }

    th {
        background-color: #2d2d2d;
    }

    input,
    textarea,
    select {
        background-color: #2d2d2d;
        color: var(--dark-color);
        border-color: #495057;
    }

    .sidebar {
        background-color: #1e1e1e;
    }

    .news-item {
        background-color: #2d2d2d;
    }
}

/* 印刷用スタイル */
@media print {
    * {
        background: transparent !important;
        color: black !important;
        box-shadow: none !important;
        text-shadow: none !important;
    }

    a,
    a:visited {
        text-decoration: underline;
    }

    a[href]:after {
        content: " (" attr(href) ")";
    }

    abbr[title]:after {
        content: " (" attr(title) ")";
    }

    .main-header,
    .sidebar,
    .main-footer {
        display: none;
    }

    .main-content {
        width: 100% !important;
        margin: 0 !important;
        padding: 0 !important;
    }

    h1, h2, h3 {
        page-break-after: avoid;
    }

    blockquote,
    tr,
    img {
        page-break-inside: avoid;
    }

    p,
    h2,
    h3 {
        orphans: 3;
        widows: 3;
    }

    thead {
        display: table-header-group;
    }
}

/* 高コントラストモード */
@media (prefers-contrast: high) {
    :root {
        --primary-color: #0000ff;
        --secondary-color: #000000;
        --success-color: #008000;
        --danger-color: #ff0000;
        --warning-color: #ffff00;
        --info-color: #00ffff;
    }

    a {
        text-decoration: underline;
    }

    button,
    .btn {
        border: 2px solid currentColor;
    }
}

/* 動きを減らす設定 */
@media (prefers-reduced-motion: reduce) {
    *,
    *::before,
    *::after {
        animation-duration: 0.01ms !important;
        animation-iteration-count: 1 !important;
        transition-duration: 0.01ms !important;
        scroll-behavior: auto !important;
    }
}

/* ==========================================================================
   カスタムプロパティのフォールバック
   ========================================================================== */

/* 古いブラウザ向けのフォールバック */
.btn-primary {
    background-color: #007bff; /* フォールバック */
    background-color: var(--primary-color);
}

.text-primary {
    color: #007bff; /* フォールバック */
    color: var(--primary-color);
}

/* ==========================================================================
   パフォーマンス最適化
   ========================================================================== */

/* GPU加速 */
.feature-card,
.btn {
    will-change: transform;
}

/* フォント表示の最適化 */
@font-face {
    font-family: 'OptimizedFont';
    src: url('font.woff2') format('woff2');
    font-display: swap;
}

/* 画像の最適化 */
img {
    max-width: 100%;
    height: auto;
    loading: lazy;
}

/* スクロールの最適化 */
html {
    scroll-behavior: smooth;
}

@supports (scroll-behavior: smooth) {
    html {
        scroll-behavior: smooth;
    }
}
